@@ -1,30 +1,9 @@
-##############################################################################
-#
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Library General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Library General Public License for more details.
-#
-#  You should have received a copy of the GNU Library General Public
-#  License along with this library; if not, write to the
-#  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-#  Boston, MA  02111-1307, USA.
-#
-#  Copyright (C) 2003 Ryan Eatmon
-#  Copyright (C) 2010-2012 Oleg G
-#
-##############################################################################
 package IO::Socket::Socks;
 
 use strict;
 use IO::Socket;
 use IO::Select;
-use Errno qw(EWOULDBLOCK EAGAIN ENOTCONN ETIMEDOUT);
+use Errno qw(EWOULDBLOCK EAGAIN ENOTCONN ETIMEDOUT ECONNABORTED);
 use Carp;
 use vars qw( @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION $SOCKS_ERROR $SOCKS5_RESOLVE $SOCKS4_RESOLVE $SOCKS_DEBUG %CODES );
 require Exporter;
@@ -71,7 +50,7 @@ use constant
 %EXPORT_TAGS = (constants => ['SOCKS_WANT_READ', 'SOCKS_WANT_WRITE', @EXPORT_OK]);
 tie $SOCKS_ERROR, 'IO::Socket::Socks::ReadOnlyVar', IO::Socket::Socks::Error->new();
 
-$VERSION = '0.63_1';
+$VERSION = '0.65_1';
 $SOCKS5_RESOLVE = 1;
 $SOCKS4_RESOLVE = 0;
 $SOCKS_DEBUG = $ENV{SOCKS_DEBUG};
@@ -157,26 +136,67 @@ use constant
 #------------------------------------------------------------------------------
 # sub new is handled by IO::Socket::INET
 #------------------------------------------------------------------------------
+###############################################################################
+#
+# new_from_fd/new_from_socket - create IO::Socket::Socks object from existing
+# non-connected socket
+###############################################################################
 sub new_from_fd
 {
     my ($class, $sock, %arg) = @_;
     
     bless $sock, $class;
     
-    my $blocking = $sock->blocking;
     $sock->autoflush(1);
-    ${*$sock}{'io_socket_timeout'} = delete $arg{Timeout};
-    
-    scalar(%arg) or return $sock;
-    if ($sock = $sock->configure(\%arg) and !defined $arg{Blocking} and !$blocking) {
-        $sock->blocking(0);
+    if (exists $arg{Timeout}) {
+        ${*$sock}{'io_socket_timeout'} = delete $arg{Timeout};
     }
     
-    return $sock;
+    scalar(%arg) or return $sock;
+    return $sock->configure(\%arg);
 }
 
 *new_from_socket = \&new_from_fd;
 
+###############################################################################
+#
+# start_SOCKS - start socks handshake on already connected socket
+###############################################################################
+sub start_SOCKS
+{
+    my ($class, $sock, %arg) = @_;
+    
+    bless $sock, $class;
+    
+    $sock->autoflush(1);
+    if (exists $arg{Timeout}) {
+        ${*$sock}{'io_socket_timeout'} = delete $arg{Timeout};
+    }
+    
+    ${*$sock}->{SOCKS} = {RequireAuth => 0};
+    
+    return $sock->command(%arg) ? $sock : undef;
+}
+
+###############################################################################
+#
+# socket - override parent socket() to prevent recreation of already created socket
+#          this is useful for new_from_fd/new_from_socket
+###############################################################################
+sub socket
+{
+    my $self = shift;
+    
+    if (-S $self) {
+        ${*$self}{'io_socket_domain'} ||= $_[0];
+        ${*$self}{'io_socket_type'}   ||= $_[1];
+        ${*$self}{'io_socket_proto'}  ||= $_[2];
+        return $self;
+    }
+    
+    return $self->SUPER::socket(@_);
+}
+
 ###############################################################################
 #
 # configure - read in the config hash and populate the object.
@@ -2062,7 +2082,7 @@ sub _fail
 {
     if(!@_ || defined($_[0]))
     {
-        $SOCKS_ERROR->set(ETIMEDOUT, $@ = 'Timeout') if $SOCKS_ERROR == undef;
+        $SOCKS_ERROR->set(ECONNABORTED, $@ = 'Socket closed by remote side') if $SOCKS_ERROR == undef;
         return;
     }
     
@@ -2324,7 +2344,9 @@ subdirectory in the distribution.
 =head3 new_from_fd($socket, %cfg)
 
 Creates a new IO::Socket::Socks client object.  new_from_socket() is the same as
-new(), but allows one to create object from an existing socket (new_from_fd is new_from_socket alias).
+new(), but allows one to create object from an existing and not connected
+(to make IO::Socket::Socks object from connected socket see "start_SOCKS")
+socket (new_from_fd is new_from_socket alias).
 Both takes the following config hash:
 
   SocksVersion => 4 or 5. Default is 5
@@ -2387,6 +2409,38 @@ The following options should be specified:
 
 Other options are facultative.
 
+=head3
+start_SOCKS($socket, %cfg)
+
+This is a class method to start socks handshake on already connected socket. This
+will bless passed $socket to IO::Socket::Socks class. %cfg is like hash in the constructor.
+Only options listed below makes sence:
+
+  Timeout
+  ConnectAddr
+  ConnectPort
+  BindAddr
+  BindPort
+  UdpAddr
+  UdpPort
+  SocksVersion
+  SocksDebug
+  SocksResolve
+  AuthType
+  RequireAuth
+  Username
+  Password
+  AuthMethods
+
+On success this method will return same $socket, but as IO::Socket::Socks object. On failure it will
+return undef (but socket will be still blessed to IO::Socket::Socks class). See example:
+
+  use IO::Socket;
+  use IO::Socket::Socks;
+  
+  my $sock = IO::Socket::INET->new("$proxy_host:$proxy_port") or die $@;
+  $sock = IO::Socket::Socks->start_SOCKS($sock, ConnectAddr => "google.com", ConnectPort => 80) or die $@;
+
 =head3
 version( )
 
